theme_colors = {
    "background_primary": "#111827",
    "background_secondary": "#1F2937",
    "background_tertiary": "#4B5563",

    "interactive_primary": "#3B82F6",
    "interactive_hover": "#60A5FA",
    "interactive_pressed": "#2563EB",
    "interactive_text": "#FFFFFF",

    "text_primary": "#F9FAFB",
    "text_secondary": "#9CA3AF",
    "text_disabled": "#6B7280",

    "success": "#10B981",
    "warning": "#F59E0B",
    "warning_hover": "#FBBF24",
    "danger": "#EF4444",
    "info": "#0EA5E9",
}